import csv
import time
import boto3 # https://aws.amazon.com/sdk-for-python/
from botocore.exceptions import ClientError
import os
import logging
from dotenv import load_dotenv

logging.basicConfig()
log = logging.getLogger("boto3")
log.setLevel(logging.DEBUG)
log.propagate = True

current_milli_time = lambda: int(round(time.time() * 1000))
dynamodb = boto3.resource(service_name='dynamodb', 
    aws_access_key_id="anything",
    aws_secret_access_key="anything",
    region_name='eu-central-1', 
    endpoint_url='http://dynamodb-local:8000/')

TABLE_NAME = "Certificates"
table = dynamodb.Table(TABLE_NAME)

try:
    table = dynamodb.create_table(
            TableName=TABLE_NAME,
            KeySchema=[
                { "AttributeName": "id", "KeyType": "HASH" },
                { "AttributeName": "cert_common_name", "KeyType": "RANGE" }
            ],
            LocalSecondaryIndexes=[
                { "IndexName": "cert_authority_index",
                "KeySchema": [
                    {
                        "AttributeName": "id",
                        "KeyType" : "HASH"
                    },
                    {
                        "AttributeName": "cert_authority",
                        "KeyType" : "RANGE"
                    }
                ],
                "Projection": {
                    "ProjectionType": "KEYS_ONLY"
                }
                },
                { "IndexName": "cert_info_link_index",
                "KeySchema": [
                        {
                            "AttributeName": "id",
                            "KeyType" : "HASH"
                        },
                        {
                            "AttributeName": "cert_info_link",
                            "KeyType" : "RANGE"
                        }
                    ],
                    "Projection": {
                        "ProjectionType": "KEYS_ONLY"
                    }
                    },
                    { "IndexName": "not_valid_before_index",
                "KeySchema": [
                        {
                            "AttributeName": "id",
                            "KeyType" : "HASH"
                        },
                        {
                            "AttributeName": "not_valid_before",
                            "KeyType" : "RANGE"
                        }
                    ],
                    "Projection": {
                        "ProjectionType": "KEYS_ONLY"
                    }
                    },
                    { "IndexName": "not_valid_after_index",
                "KeySchema": [
                        {
                            "AttributeName": "id",
                            "KeyType" : "HASH"
                        },
                        {
                            "AttributeName": "not_valid_after",
                            "KeyType" : "RANGE"
                        }
                    ],
                    "Projection": {
                        "ProjectionType": "KEYS_ONLY"
                    }
                    },
                    { "IndexName": "log_type_index",
                "KeySchema": [
                        {
                            "AttributeName": "id",
                            "KeyType" : "HASH"
                        },
                        {
                            "AttributeName": "log_type",
                            "KeyType" : "RANGE"
                        }
                    ],
                    "Projection": {
                        "ProjectionType": "KEYS_ONLY"
                    }
                    }
            ],
            AttributeDefinitions=[
                { "AttributeName": "id", "AttributeType": "N" },
                { "AttributeName": "cert_info_link", "AttributeType": "S" },
                { "AttributeName": "not_valid_before", "AttributeType": "N" },
                { "AttributeName": "not_valid_after", "AttributeType": "N" },
                { "AttributeName": "cert_common_name", "AttributeType": "S" },
                { "AttributeName": "cert_authority", "AttributeType": "S" },
                { "AttributeName": "log_type", "AttributeType": "S" }
            ],
            ProvisionedThroughput={
                "ReadCapacityUnits": 1,
                "WriteCapacityUnits": 1
            }
    )
except ClientError:
    print("Table already exists")
    pass

# Wait until the table exists.
table.meta.client.get_waiter('table_exists').wait(TableName='Certificates')

def add_rows_in_batch(data):
    with table.batch_writer() as batch:
        for item in data:
            batch.put_item(Item = item)


def run_batch_migration():
    start = current_milli_time()
    row_count = 0
    batch = []
    with open("./sample.csv", newline = '') as csvfile:
        reader = csv.reader(csvfile, delimiter = ';')
        for row in reader:
            print('Adding ', row[3])
            row_count += 1
            item = {
                'id': int(row_count),
                'cert_info_link': row[0],
                'not_valid_before': int(row[1]),
                'not_valid_after': int(row[2]),
                'cert_common_name': row[3],
                'cert_authority': row[4],
                'log_type': row[5]
            }
            batch.append(item)
            if row_count % 25 == 0:
                add_rows_in_batch(batch)
                batch = []

    print('Number of rows processed - ', str(row_count))
    end = current_milli_time()
    print('Total time taken for migration : ', str((end - start) / 1000), ' secs')


if __name__ == "__main__":
    run_batch_migration()
    
# https://stackoverflow.com/questions/32678325/how-to-import-bulk-data-from-csv-to-dynamodb